/* $Id: sstream.c,v 1.7 2001/12/20 23:03:02 aa5779 Exp $ */
/*
	Copyright (C) 2001 Artem V. Andreev (artem@AA5779.spb.edu)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <string.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include "util.h"
#include "interp.h"
#include "bstream.h"
#define SSTREAM_IMP
#include "sstream.h"

stream_class script_stream_class = {"script", ss_open, ss_close, ss_read, ss_writestr, 
								   ss_writetag, ss_ioctl, ss_getpos,
								   ss_setpos, ss_rewind};

typedef struct sstream_data
{
	dstring *id;
	interpreter *i;
} sstream_data;

static dstring *get_stream_name(sstream_data *ssd)
{
	return ssd->id;
}
	
static int stream_counter = 0;

stream *ss_open(interpreter *i, char *name, char *sname, int writeable)
{
	stream *ns = ALLOC(stream);
	char *arg;

	sname = xstrdup(sname);
	arg = strchr(sname, '?');
	if(arg)
	{
		*arg = '\0';
		arg++;
	}
	ns->class = &script_stream_class;
	ns->direction = stream_out;
	ns->data = ALLOC(sstream_data);
	((sstream_data *)ns->data)->id = ds_fix(ds_append(ds_createch('s'), ds_fromint(stream_counter++, 10, 0)));
	((sstream_data *)ns->data)->i = find_subint(i, sname);
	discard_value(eval_procedurev(((sstream_data *)ns->data)->i, "streamopen", 
				get_stream_name(ns->data), ds_create(arg), ds_create(writeable ? "true" : ""), NULL));
	free(sname);
	return ns;
}

void ss_close(interpreter *i, stream *s)
{
	sstream_data *ssd = s->data;
	discard_value(eval_procedurev(ssd->i, "streamclose", get_stream_name(ssd), NULL));
	ds_unfix(ssd->id);
	FREE(ssd);
}

html_chunk *ss_read(interpreter *i, stream *s)
{
	sstream_data *ssd = s->data; 
	discard_value(eval_procedurev(ssd->i, "read", get_stream_name(ssd), NULL));
	if(!ssd->i->this_chunk || HTML_CHUNK_TYPE(ssd->i->this_chunk) == hct_none)
		return NULL;
	return ssd->i->this_chunk;
}

void ss_writestr(interpreter *i, stream *s, char *str)
{
	sstream_data *ssd = s->data;
	discard_value(eval_procedurev(ssd->i, "writestr", get_stream_name(ssd), ds_create(str), NULL));
}

void ss_writetag(interpreter *i, stream *s, html_chunk *ch)
{
	sstream_data *ssd = s->data;
	release_html_chunk(ssd->i->this_chunk);
	ssd->i->this_chunk = copy_html_chunk(ch);
	discard_value(eval_procedurev(ssd->i, "writetag", get_stream_name(ssd), NULL));
}

char *ss_ioctl(interpreter *i, stream *s, char *ctlstr)
{
	static char buffer[256];
	sstream_data *ssd = s->data;
	dstring *ds;
		
	if(ctlstr)
	{
		if(!strcasecmp(ctlstr, "providesbasic"))
			return "";
	}
	ds = eval_procedurev(ssd->i, "ioctl", get_stream_name(ssd), ds_create(ctlstr), NULL);
	if(!ds)
		return "";
	if(!strcmp(DS_BODY(ds), "not-implemented"))
		return NULL;
	strncpy(buffer, DS_BODY(ds), sizeof(buffer) - 1);
	ds_unfix(ds);
	return buffer;
}

int ss_getpos(interpreter *i, stream *s)
{
	sstream_data *ssd = s->data;
	dstring *ds;
	
	ds = eval_procedurev(ssd->i, "getpos", get_stream_name(ssd), NULL);
	if(!ds)
		return -1;
	else
	{
		ds_unfix(ds);
		return convert_to_int(DS_BODY(ds));
	}
}

void ss_setpos(interpreter *i, stream *s, int pos)
{
	sstream_data *ssd = s->data;
	
	discard_value(eval_procedurev(ssd->i, "setpos", get_stream_name(ssd), ds_fromint(pos, 10, 0), NULL));
}

void ss_rewind(interpreter *i, stream *s)
{
	sstream_data *ssd = s->data;
	discard_value(eval_procedurev(ssd->i, "rewind", get_stream_name(ssd), NULL));
}
